// LCD_I2C_SAINSMART_DRIVER FOR MICROCHIP PIC
// Written for PIC18F4550
// Commands based on CCS compiler 4.141
// REV-0
// 04 May 2013
// I2C LCD Address (0x3F in this driver) can be changed below (e.g. to 0x27) if required
//
// WRITES to 20x4 LCD marked J204A using Sainsmart I2C board marked LCD2004
// Does not READ
//
// Note: Regarding the Sainsmart I2C adapter, output-P3 (pin-7) from IC 
//       PCF8574 is wired such that if LOW, it shuts off
//       the LCD's LED backlight.  P3 doesn't seem to affect any other 
//       operations since P3 is not wired to any of the HD44780 
//       data inputs.  Therefore P3 must be HIGH on all write operations. 
//
//    P7 - D7        P3 - BACKLIGHT LED
//    P6 - D6        P2 - E
//    P5 - D5        P1 - R/W
//    P4 - D4        P0 - RS
//


// Enter LCD address given by manufacturer below
#define LCD2004_addr 0x3F 

// Following line creates appropriate address byte for I2C interface (shift left 1-bit)
#define LCD2004_I2C_ADDR (LCD2004_addr<<1)

// Following line tested for CCS compiler and best goes into main program body
// #use i2c(MASTER, FAST, SCL=PIN_A3, SDA=PIN_A2, FORCE_SW, STREAM=LCDI2C)
// Note: Using PICs with HW I2C capability can increase speed up to 2x over CCS SW I2C

// Line addresses for LCDs which use
// the Hitachi HD44780U controller chip
#define LCD2004_LINE_1_ADDRESS 0x00
#define LCD2004_LINE_2_ADDRESS 0x40
#define LCD2004_LINE_3_ADDRESS 0x14
#define LCD2004_LINE_4_ADDRESS 0x54

// Line addresses for LCDs which use
// the Hitachi HD66712U controller chip
/*
#define LCD2004_LINE_1_ADDRESS 0x00
#define LCD2004_LINE_2_ADDRESS 0x20
#define LCD2004_LINE_3_ADDRESS 0x40
#define LCD2004_LINE_4_ADDRESS 0x60
*/

#separate
void LCD_I2C_Test();

#separate
void LCD2004_I2C_Write(char text[]); 

#separate
void LCD2004_I2C_Init();

#separate
void LCD2004_I2C_Gotoxy(int8 x, int8 y);

#separate
void LCD2004_I2C_Putc(char c);

#separate
void LCD2004_I2C_Send_Byte(int8 address, int8 n);

int lcd_j,lcd_max,lcd_tempbyte;
int lcd_fixed_delay=1;  // Fixed delay in msec to allow LCD to process I2C instruction
int LCD_RS_State;       // HD44780U LCD Instruction Type: COMMAND=0, DATA=1
int lcd2004_line;
unsigned char lcd_buffer[20];

//===================================
void LCD_I2C_Test() 
{
repeat:
 
   LCD2004_I2C_gotoxy(10, 1);
   strcpy(lcd_buffer,"HELLO WORLD");   // copy name of project to string
   LCD2004_I2C_Write(lcd_buffer);      // write name of project to LCD

   LCD2004_I2C_gotoxy(10, 3);
   strcpy(lcd_buffer,"HELLO WORLD");   // copy name of project to string
   LCD2004_I2C_Write(lcd_buffer);      // write name of project to LCD

   LCD2004_I2C_gotoxy(1, 2);
   printf(LCD2004_I2C_putc,"*");

   LCD2004_I2C_gotoxy(1, 4);
   printf(LCD2004_I2C_putc,"*");
   
   delay_ms(1000);
   LCD2004_I2C_Send_Byte(0, 1);        // Clear screen

   LCD2004_I2C_gotoxy(1, 2);
   strcpy(lcd_buffer,"HELLO WORLD");   // copy name of project to string
   LCD2004_I2C_Write(lcd_buffer);      // write name of project to LCD

   LCD2004_I2C_gotoxy(1, 4);
   strcpy(lcd_buffer,"HELLO WORLD");   // copy name of project to string
   LCD2004_I2C_Write(lcd_buffer);      // write name of project to LCD

   LCD2004_I2C_gotoxy(20, 1);
   printf(LCD2004_I2C_putc,"*");
     
   LCD2004_I2C_gotoxy(20, 3);
   printf(LCD2004_I2C_putc,"*");
   
   delay_ms(1000);
   LCD2004_I2C_Send_Byte(0, 1);        // Clear screen
 
goto repeat;
}


//===================================
void LCD2004_I2C_Write(char text[]) 
{
//function to write a string to LCD via I2C
   i2c_start(LCDI2C);
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,LCD2004_I2C_ADDR);
   delay_ms(lcd_fixed_delay);
   lcd_max=strlen(text);
   for(lcd_j=0; lcd_j<lcd_max; ++lcd_j)
   {
   // Send upper nibble
      lcd_tempbyte=text[lcd_j]&0xF0;          // Write Most Significant Nibble of text[j]  (LCD in 4-bit mode)
      i2c_write(LCDI2C,lcd_tempbyte|0x09);    // Least Significant Nibbles for I2C data = 9-D-9 for Data
      i2c_write(LCDI2C,lcd_tempbyte|0x0D);
      i2c_write(LCDI2C,lcd_tempbyte|0x09);
      
   // Send lower nibble
      lcd_tempbyte=text[lcd_j]<<4;            // Write Most Significant Nibble of text[j]  (LCD in 4-bit mode)
      i2c_write(LCDI2C,lcd_tempbyte|0x09);    // Least Significant Nibbles for I2C data = 9-D-9 for Data
      i2c_write(LCDI2C,lcd_tempbyte|0x0D);
      i2c_write(LCDI2C,lcd_tempbyte|0x09);
   }
   i2c_stop(LCDI2C);
   delay_ms(lcd_fixed_delay);
}

//===================================
void LCD2004_I2C_Gotoxy(int8 x, int8 y)
{
int8 address;

   switch(y)
     {
      case 1:
        address = LCD2004_LINE_1_ADDRESS;
        break;
   
      case 2:
        address = LCD2004_LINE_2_ADDRESS;
        break;
   
      case 3:
        address = LCD2004_LINE_3_ADDRESS;
        break;
   
      case 4:
        address = LCD2004_LINE_4_ADDRESS;
        break;
   
      default:
        address = LCD2004_LINE_1_ADDRESS;
        break;     
     }

   address += x-1;
   LCD2004_I2C_Send_Byte(0, 0x80 | address);
}

//===================================
void LCD2004_I2C_Putc(char c)
{
 switch(c)
   {
    case '\f':
      LCD2004_I2C_Send_Byte(0x00,0x01);
      lcd2004_line = 1;
      delay_ms(lcd_fixed_delay);
      break;
   
    case '\n':
       LCD2004_I2C_Send_Byte(0x01, ++lcd2004_line);
       break;
   
    case '\b':
       LCD2004_I2C_Send_Byte(0x00,0x10);
       break;
   
    default:
       LCD2004_I2C_Send_Byte(0x01,c);
       break;
   }
}

//===================================
void LCD2004_I2C_Send_Byte(int8 address, int8 n)
{
   if(address)
      LCD_RS_State=1;   // data
   else
      LCD_RS_State=0;   // command
        
   i2c_start(LCDI2C);
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,LCD2004_I2C_ADDR);
   delay_ms(lcd_fixed_delay);
   
   // Send upper nibble
   lcd_tempbyte=(n&0xF0)|LCD_RS_State;    // Write Most Significant Nibble of text[j]  (LCD in 4-bit mode)
   i2c_write(LCDI2C,lcd_tempbyte|0x08);   // Least Significant Nibbles for I2C DATA = 9-D-9 (LSB=1)
   i2c_write(LCDI2C,lcd_tempbyte|0x0C);
   i2c_write(LCDI2C,lcd_tempbyte|0x08);
   
   // Send lower nibble
   lcd_tempbyte=(n<<4)|LCD_RS_State;      // Write Most Significant Nibble of text[j]  (LCD in 4-bit mode)
   i2c_write(LCDI2C,lcd_tempbyte|0x08);   // Least Significant Nibbles for I2C DATA = 9-D-9 (LSB=1)
   i2c_write(LCDI2C,lcd_tempbyte|0x0C);
   i2c_write(LCDI2C,lcd_tempbyte|0x08);
   
   i2c_stop(LCDI2C);
   delay_ms(lcd_fixed_delay);
}

//===================================
void LCD2004_I2C_Init()
{  
// Initialization commands for sainsmart LCD2004 via I2C
   delay_ms(50);               // Let LCD power up

   i2c_start(LCDI2C);          // Claim I2C BUS
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,LCD2004_I2C_ADDR); // Tell all I2C devices you are talking to LCD2004_I2C_ADDR
   delay_ms(lcd_fixed_delay);
   
   i2c_write(LCDI2C,0x38);     // Write Nibble 0x03 (per HD44780U initialization spec)
   delay_ms(lcd_fixed_delay);  // Least Significant Nibbles for I2C COMMAND = 8-C-8 (LSB=0)
   i2c_write(LCDI2C,0x3C);     
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x38);   
   delay_ms(lcd_fixed_delay);
   
   i2c_write(LCDI2C,0x38);     // Write Nibble 0x03 (per HD44780U initialization spec)
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x3C);  
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x38);  
   delay_ms(lcd_fixed_delay);
   
   i2c_write(LCDI2C,0x38);     // Write Nibble 0x03 (per HD44780U initialization spec)
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x3C);  
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x38);  
   delay_ms(lcd_fixed_delay);
   
   i2c_write(LCDI2C,0x28);     // Write Nibble 0x02 (per HD44780U initialization spec)   
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x2C);      
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x28);   
   delay_ms(lcd_fixed_delay);
   
   i2c_write(LCDI2C,0x28);     // Set mode: 4-bit, 2+lines, 5x8 dots   
   delay_ms(lcd_fixed_delay);  // Write Byte 0x28 
   i2c_write(LCDI2C,0x2C);      
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x28);   
   delay_ms(lcd_fixed_delay);
   
   i2c_write(LCDI2C,0x88);    
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x8C);     
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x88);     
   delay_ms(lcd_fixed_delay);
   
   i2c_write(LCDI2C,0x08);     // Display ON: Write Byte 0x0C 
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x0C);
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x08);
   delay_ms(lcd_fixed_delay);
   
   i2c_write(LCDI2C,0xC8);
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0xCC);
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0xC8);
   delay_ms(lcd_fixed_delay);
   
   i2c_write(LCDI2C,0x08);     // Clear Display: Write Byte 0x01 
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x0C);
   delay_ms(lcd_fixed_delay);   
   i2c_write(LCDI2C,0x08);
   delay_ms(lcd_fixed_delay);
   
   i2c_write(LCDI2C,0x18);
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x1C);
   delay_ms(lcd_fixed_delay);   
   i2c_write(LCDI2C,0x18);
   delay_ms(lcd_fixed_delay);
   
   i2c_write(LCDI2C,0x08);     // Increment cursor: Write Byte 0x06
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x0C);
   delay_ms(lcd_fixed_delay);   
   i2c_write(LCDI2C,0x08);
   delay_ms(lcd_fixed_delay);

   i2c_write(LCDI2C,0x68);
   delay_ms(lcd_fixed_delay);
   i2c_write(LCDI2C,0x6C);
   delay_ms(lcd_fixed_delay);   
   i2c_write(LCDI2C,0x68);
   delay_ms(lcd_fixed_delay);
   
   i2c_stop(LCDI2C);          // Terminate I2C transfer
   delay_ms(lcd_fixed_delay);
}
















